import base64

import getpass
import socket
import string

import os
import subprocess

import time
import datetime

import inspect


def find_exe(name):
    """
    Find an executable with the given name.
    :param name:
    :return:
    """
    for path in os.getenv('PATH').split(os.pathsep):
        for ext in ('', '.exe', '.cmd', '.bat', '.sh'):
            full_path = os.path.join(path, name + ext)
            if os.path.isfile(full_path) and os.access(full_path, os.X_OK):
                return full_path
    return None


def generate_version_file(infile='version.py.in', outfile='version.py', **kwargs):
    """
    Using input file, generate an output file with the version info for this package.
    This should generally be used in setup.py to generate build stamp info.
    :param infile:
    :param outfile:
    :param kwargs:
    :return:
    """

    if not os.path.exists(outfile) and os.path.exists(infile):
        now = datetime.datetime.now()
        repo_hash = ''
        git_path = find_exe('git')
        if git_path is not None:
            git_cmd = [find_exe('git'), 'log', '--format=%h', '-1']
            repo_hash = subprocess.Popen(git_cmd, stdout=subprocess.PIPE).communicate()[0].strip()
        if repo_hash.strip() == '':
            repo_hash = '{now.hour}.{now.minute}.{now.second}'.format(now=now)

        sub_dict = {
            'repo_hash': repo_hash,
            'build_date': '{now.year}.{now.month}.{now.day}'.format(now=now),
            'build_time': '{now.hour}.{now.minute}.{now.second}'.format(now=now),
            'build_host': socket.gethostname(),
            'build_user': getpass.getuser()
        }
        sub_dict.update(**kwargs)

        with open(infile) as version_in:
            version_data = version_in.read()
            version_final = string.Template(version_data).safe_substitute(sub_dict)

        with open(outfile, 'w') as version_out:
            version_out.write(version_final)

    return outfile if os.path.exists(outfile) else None


def generateCookieSecret(path):
    secret = base64.b64encode(os.urandom(32))
    with open(path, 'w') as fp:
        fp.write(secret)
    return secret


def getParent():
    # inspect who called this function
    frames = inspect.getouterframes(inspect.currentframe())
    # get the caller frame
    frame = frames[-1]
    # get the path of the caller
    script = os.path.abspath(frame[1])

    return script



#!/usr/bin/env python

import os

from distutils.core import setup

import pyaas.skel

setup(
    name='example',
    author='Your Name Here',
    author_email='your@email',
    long_description=open('README.rst').read(),
    version='0.1',
    description='Example Project',
    license='TBD',
    packages=[
        'example',
        'example.handlers',
    ],
    scripts=[
        'bin/example.py',
    ],
    data_files=pyaas.skel.add_data_files('etc', 'share', 'var'),
    install_requires=[
        'pyaas >= 0.5.3',
    ],
    zip_safe=False
)




import json
import collections
import abc

import pyaas

__all__ = ['Records', 'Record']

class Instance(type):
    def __getattr__(cls, key):
        try:
            return object.__getattribute__(cls, key)
        except AttributeError:
            try:
                cls.instance = pyaas.storage.databases[key]
                return cls
            except KeyError:
                raise AttributeError(key)

    def __getitem__(cls, key):
        # TODO: set instance to an actual database class instance
        try:
            cls.instance = pyaas.storage.databases[key]
            return cls
        except KeyError:
            raise AttributeError(key)

#    @property
#    def instance(self):
#        return self.instance
#
#    @instance.setter
#    def instance(self, value):
#        self.instance = value


class AbstractIntsance(Instance, abc.ABCMeta):
    'combined metaclass to support MutableMapping'


class Records(object):
    __metaclass__ = Instance

    def __init__(self, records):
        self.instance = self.__class__.instance
        self.records = records

    @classmethod
    def Read(cls, params=None, sort=None):
        return cls(cls.instance.Find(cls.RECORD.table(), params, sort))

    def Delete(self):
        for record in self.records:
            self.RECORD(record).Delete()
        pass

    @classmethod
    def Count(cls):
        return cls.instance.Count(cls.RECORD.table())

# List methods to iterate over the collection

    def __len__(self):
        return len(self.records)

    def __iter__(self):
        for record in self.records:
            yield self.RECORD(record)

    def __getitem__(self, idx):
        return self.records.__getitem__(idx)

    @property
    def json(self):
        return json.dumps([dict(e) for e in self.records], separators=(',', ':'))


class Record(collections.MutableMapping):
    __metaclass__ = AbstractIntsance

    TABLE_NAME = None
    ID_COLUMN = 'id'

    def __init__(self, record):
        self.instance = self.__class__.instance

        try:
            self.id = record[self.ID_COLUMN]
        except KeyError:
            self.id = None

        self.record = dict(record)
        self.Init()

    def Init(self):
        pass

    @classmethod
    def table(cls):
        return cls.__name__ if cls.TABLE_NAME is None else cls.TABLE_NAME

# CRUD methods

    @classmethod
    def Create(cls, values):
        return cls(values).Insert()

    def Insert(self):
        self.instance.Insert(self.table(), self.record)
        if self.id is None:
            self.id = self.record[self.ID_COLUMN]
        return self

    @classmethod
    def Read(cls, _id):
        record = cls.instance.FindOne(cls.table(), _id, cls.ID_COLUMN)
        return cls(record) if record else None

    def Update(self, values=None):
        if values:
            self.record.update(values)
        self.instance.Update(self.table(), self.record, self.ID_COLUMN)
        return self

    def Delete(self):
        self.instance.Remove(self.table(), self.id, self.ID_COLUMN)
        return True

# Convenience function to access data as JSON

    @property
    def json(self):
        return json.dumps(dict(self.record), separators=(',', ':'))

# Direct access of records

    def __getattr__(self, attr):
        try:
            return object.__getattribute__(self, attr)
        except AttributeError:
            if attr in self.record:
                return self.record[attr]
            raise

    def __iter__(self):
        return self.record.__iter__

    def __len__(self):
        return len(self.record)

# Dictionary methods to access the data

    def __getitem__(self, key):
        return self.record.__getitem__(key)

    def __setitem__(self, key, value):
        return self.record.__setitem__(key, value)

    def __delitem__(self, key):
        return self.record.__delitem__(key)

    def keys(self):
        return self.record.keys()
